<?php

namespace ProveSource\SocialProof\Observer;

use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Model\Session;
use Magento\Sales\Model\Order;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\HTTP\Client\Curl;
use ProveSource\SocialProof\Model\SettingsEntryFactory;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use \Psr\Log\LoggerInterface;
use Magento\Store\Model\StoreManagerInterface;

class PurchaseObserver implements ObserverInterface
{
    protected $curl;
    protected $customerSession;
    protected $customerrepository;
    protected $order;
    protected $settings;
    protected $apiKey;
    protected $remoteAddress;
    protected $logger;
    protected $storeManager;

    /**
     * Constructor.
     */
    public function __construct(
        Curl $curl,
        Session $customerSession,
        CustomerRepositoryInterface $customerrepository,
        Order $order,
        SettingsEntryFactory $settings,
        RemoteAddress $remoteAddress,
        LoggerInterface $logger,
        StoreManagerInterface $storeManager)
    {
        $this->curl = $curl;
        $this->customerSession = $customerSession;
        $this->customerrepository = $customerrepository;
        $this->order = $order;
        $this->settings = $settings;
        $this->apiKey = $this->settings->create()->load('apiKey', 'key')->getValue();
        $this->remoteAddress = $remoteAddress;
        $this->logger = $logger;
        $this->storeManager = $storeManager;
    }

    /**
     * Execute observer action.
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        try {
            $this->logger->info('[ProveSource/purchaseObserver] purchase event');
            $apiKey = $this->apiKey;
            if (!isset($this->apiKey) || empty($this->apiKey)) {
                $this->apiKey = $this->settings->create()->load('apiKey', 'key')->getValue();
                $apiKey = $this->apiKey;
            }
            if (!$apiKey) {
                $this->logger->info('[ProveSource/purchaseObserver] bad API key, aborting...');
                return;
            }

            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $currencysymbol = $objectManager->get('Magento\Store\Model\StoreManagerInterface');
            $currency = $currencysymbol->getStore()->getCurrentCurrencyCode();

            $order = $observer->getEvent()->getOrder();
            $ip = $order->getRemoteIp();
            $ips = [$ip, $order->getXForwardedFor()];
            if(!empty($this->remoteAddress)) {
                array_push($ips, $this->remoteAddress->getRemoteAddress());
            }

            //customer info
            $firstname = $order->getBillingAddress()->getFirstName();
            $lastname = $order->getBillingAddress()->getLastName();
            $email = $order->getCustomerEmail();

            //cart info
            $total = $order->getGrandTotal();
            $product = $order->getAllItems()[0];
            $productData = $objectManager
                ->get('Magento\Catalog\Model\Product')
                ->load($product->getProductId());
            $store = $objectManager
                ->get('Magento\Store\Model\StoreManagerInterface')
                ->getStore($order->getStore()->getId());
            $id = $product->getProductId();
            $quantity = $product->getQtyOrdered();
            $price = $product->getPrice();
            $name = $product->getName();
            $link = $productData->getProductUrl();
            $image = $productData->getImage();
            if (strpos($image, 'http') === false) {
                $image = $store->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA)
                .'catalog/product/'
                .$image;
            }
            $siteUrl = $store->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);

            $product = [
                'id' => $id,
                'quantity' => (int) $quantity,
                'price' => (float) $price,
                'name' => $name,
                'link' => $link,
                'image' => $image,
            ];
            $orderinfo = [
                'firstName' => $firstname,
                'lastName' => $lastname,
                'email' => $email,
                'total' => (float) $total,
                'currency' => $currency,
                'ip' => $ip,
                'ips' => $ips,
                'products' => [$product],
                'siteUrl' => $siteUrl,
                'store' => [
                    'id' => $this->storeManager->getStore()->getId(),
                    'code' => $this->storeManager->getStore()->getCode(),
                    'url' => $this->storeManager->getStore()->getCurrentUrl(),
                    'name' => $this->storeManager->getStore()->getName(),
                    'websiteId' => $this->storeManager->getStore()->getWebsiteId(),
                ]
            ];

            $data = json_encode($orderinfo);
            $url = 'https://api.provesrc.com/webhooks/track/magento2';

            $this->logger->info('[ProveSource/purchaseObserver] sending purchase data');

            $this->curl->addHeader('Content-type', 'application/json');
            $this->curl->addHeader('Authorization', 'Bearer '.$apiKey);
            $this->curl->post($url, $data);

            $this->logger->info('[ProveSource/purchaseObserver] purchase data sent');

            return $this->curl->getBody();
        } catch (\Exception $e) {
            $this->logger->error('[ProveSource/purchaseObserver] failed to process order', ['error' => $e]);
        }

        return null;
    }
}
